@@ -4,6 +4,8 @@
 #include "AppConfig.h"
 #include "CubicSDR.h"
 
+#include <wx/msgdlg.h>
+
 DeviceConfig::DeviceConfig() : deviceId("") {
 	ppm.store(0);
 	offset.store(0);
@@ -39,6 +41,14 @@ long DeviceConfig::getSampleRate() {
     return sampleRate.load();
 }
 
+void DeviceConfig::setAntennaName(const std::string& name) {
+    antennaName = name;
+}
+
+const std::string& DeviceConfig::getAntennaName() {
+    return antennaName;
+}
+
 void DeviceConfig::setAGCMode(bool agcMode) {
     this->agcMode.store(agcMode);
 }
@@ -81,7 +91,9 @@ std::string DeviceConfig::getDeviceName() {
 }
 
 void DeviceConfig::save(DataNode *node) {
+
     std::lock_guard < std::mutex > lock(busy_lock);
+    
     *node->newChild("id") = deviceId;
     *node->newChild("name") = deviceName;
     *node->newChild("ppm") = ppm.load();
@@ -89,6 +101,10 @@ void DeviceConfig::save(DataNode *node) {
     *node->newChild("sample_rate") = sampleRate.load();
     *node->newChild("agc_mode") = agcMode.load()?1:0;
 
+    if (!antennaName.empty()) {
+        *node->newChild("antenna") = antennaName;
+    }
+
     if (streamOpts.size()) {
         DataNode *streamOptsNode = node->newChild("streamOpts");
         for (ConfigSettings::const_iterator opt_i = streamOpts.begin(); opt_i != streamOpts.end(); opt_i++) {
@@ -149,6 +165,12 @@ void DeviceConfig::load(DataNode *node) {
         sample_rate_node->element()->get(sampleRateValue);
         setSampleRate(sampleRateValue);
     }
+    if (node->hasAnother("antenna")) {
+        DataNode *antenna_node = node->getNext("antenna");
+        std::string antennaNameValue;
+        antenna_node->element()->get(antennaNameValue);
+        setAntennaName(antennaNameValue);
+    }
     if (node->hasAnother("streamOpts")) {
         DataNode *streamOptsNode = node->getNext("streamOpts");
         for (int i = 0, iMax = streamOptsNode->numChildren(); i<iMax; i++) {
@@ -485,6 +507,51 @@ bool AppConfig::getBookmarksVisible() {
     return bookmarksVisible.load();
 }
 
+void AppConfig::setRecordingPath(std::string recPath) {
+    recordingPath = recPath;
+}
+
+std::string AppConfig::getRecordingPath() {
+    return recordingPath;
+}
+
+bool AppConfig::verifyRecordingPath() {
+    string recPathStr = wxGetApp().getConfig()->getRecordingPath();
+    
+    if (recPathStr.empty()) {
+        wxMessageBox( wxT("Recording path is not set.  Please use 'Set Recording Path' from the 'File' Menu."), wxT("Recording Path Error"), wxICON_INFORMATION);
+        
+        return false;
+    }
+    
+    wxFileName recPath(recPathStr);
+    
+    if (!recPath.Exists() || !recPath.IsDirWritable()) {
+        wxMessageBox( wxT("Recording path does not exist or is not writable.  Please use 'Set Recording Path' from the 'File' Menu."), wxT("Recording Path Error"), wxICON_INFORMATION);
+        
+        return false;
+    }
+    
+    return true;
+}
+
+
+void  AppConfig::setRecordingSquelchOption(int enumChoice) {
+	recordingSquelchOption = enumChoice;
+}
+
+int  AppConfig::getRecordingSquelchOption() {
+	return recordingSquelchOption;
+}
+
+void  AppConfig::setRecordingFileTimeLimit(int nbSeconds) {
+	recordingFileTimeLimitSeconds = nbSeconds;
+}
+
+int  AppConfig::getRecordingFileTimeLimit() {
+	return recordingFileTimeLimitSeconds;
+}
+
 
 void AppConfig::setConfigName(std::string configName) {
     this->configName = configName;
@@ -539,6 +606,12 @@ bool AppConfig::save() {
         *window_node->newChild("bookmark_visible") = bookmarksVisible.load();
     }
     
+	//Recording settings:
+    DataNode *rec_node = cfg.rootNode()->newChild("recording");
+    *rec_node->newChild("path") = recordingPath;
+	*rec_node->newChild("squelch") = recordingSquelchOption;
+	*rec_node->newChild("file_time_limit") = recordingFileTimeLimitSeconds;
+    
     DataNode *devices_node = cfg.rootNode()->newChild("devices");
 
     std::map<std::string, DeviceConfig *>::iterator device_config_i;
@@ -721,6 +794,26 @@ bool AppConfig::load() {
         }
     }
     
+	//Recording settings:
+    if (cfg.rootNode()->hasAnother("recording")) {
+        DataNode *rec_node = cfg.rootNode()->getNext("recording");
+
+        if (rec_node->hasAnother("path")) {
+            DataNode *rec_path = rec_node->getNext("path");
+            recordingPath = rec_path->element()->toString();
+        }
+
+		if (rec_node->hasAnother("squelch")) {
+			DataNode *rec_squelch = rec_node->getNext("squelch");
+			rec_squelch->element()->get(recordingSquelchOption);
+		}
+
+		if (rec_node->hasAnother("file_time_limit")) {
+			DataNode *rec_file_time_limit = rec_node->getNext("file_time_limit");
+			rec_file_time_limit->element()->get(recordingFileTimeLimitSeconds);
+		}
+    }
+    
     if (cfg.rootNode()->hasAnother("devices")) {
         DataNode *devices_node = cfg.rootNode()->getNext("devices");
 
