@@ -4,6 +4,7 @@
 #include "BookmarkMgr.h"
 #include "CubicSDR.h"
 #include "DataTree.h"
+#include <wx/string.h>
 
 #define BOOKMARK_RECENTS_MAX 25
 
@@ -18,7 +19,8 @@ BookmarkMgr::BookmarkMgr() {
 //represents an empty BookMarkList that is returned by reference by some functions.
 const BookmarkList BookmarkMgr::emptyResults;
 
-void BookmarkMgr::saveToFile(std::string bookmarkFn, bool backup) {
+void BookmarkMgr::saveToFile(std::string bookmarkFn, bool backup, bool useFullpath) {
+
     DataTree s("cubicsdr_bookmarks");
     DataNode *header = s.rootNode()->newChild("header");
     header->newChild("version")->element()->set(wxString(CUBICSDR_VERSION).ToStdWstring());
@@ -48,7 +50,21 @@ void BookmarkMgr::saveToFile(std::string bookmarkFn, bool backup) {
         *group->newChild("@expanded") = (getExpandState(bmd_i.first)?std::string("true"):std::string("false"));
 
         for (auto &bm_i : bmd_i.second ) {
-            group->newChildCloneFrom("modem", bm_i->node);
+
+			//if a matching demodulator exists, use its data instead to be be saved, because output_device could have been
+			//modified by the user. So, save that "live" version instead.
+			auto matchingDemod = wxGetApp().getDemodMgr().getLastDemodulatorWith(bm_i->type,
+				bm_i->label,
+				bm_i->frequency,
+				bm_i->bandwidth);
+
+			if (matchingDemod != nullptr) {
+
+				wxGetApp().getDemodMgr().saveInstance(group->newChild("modem"), matchingDemod);
+			}
+			else {
+				group->newChildCloneFrom("modem", bm_i->node);
+			}
         }
     }
 
@@ -62,9 +78,18 @@ void BookmarkMgr::saveToFile(std::string bookmarkFn, bool backup) {
         recent_modems->newChildCloneFrom("modem", r_i->node);
     }
 
-    wxFileName saveFile(wxGetApp().getConfig()->getConfigDir(), bookmarkFn);
-    wxFileName saveFileBackup(wxGetApp().getConfig()->getConfigDir(), bookmarkFn + ".backup");
-    
+	wxFileName saveFile;
+	wxFileName saveFileBackup;
+
+	if (useFullpath) {
+		saveFile.Assign(bookmarkFn);
+		saveFileBackup.Assign(bookmarkFn + ".backup");
+	}
+	else {
+		saveFile.Assign(wxGetApp().getConfig()->getConfigDir(), bookmarkFn);
+		saveFileBackup.Assign(wxGetApp().getConfig()->getConfigDir(), bookmarkFn + ".backup");
+	}
+
     if (saveFile.IsDirWritable()) {
         // Hopefully leave at least a readable backup in case of failure..
         if (backup && saveFile.FileExists() && (!saveFileBackup.FileExists() || saveFileBackup.IsFileWritable())) {
@@ -74,20 +99,28 @@ void BookmarkMgr::saveToFile(std::string bookmarkFn, bool backup) {
     }
 }
 
-bool BookmarkMgr::loadFromFile(std::string bookmarkFn, bool backup) {
-    wxFileName loadFile(wxGetApp().getConfig()->getConfigDir(), bookmarkFn);
-    wxFileName failFile(wxGetApp().getConfig()->getConfigDir(), bookmarkFn + ".failedload");
-    wxFileName lastLoaded(wxGetApp().getConfig()->getConfigDir(), bookmarkFn + ".lastloaded");
-    wxFileName backupFile(wxGetApp().getConfig()->getConfigDir(), bookmarkFn + ".backup");
+bool BookmarkMgr::loadFromFile(std::string bookmarkFn, bool backup, bool useFullpath) {
+
+	wxFileName loadFile;
+	wxFileName failFile;
+	wxFileName lastLoaded;
+	wxFileName backupFile;
+
+	if (useFullpath) {
+		loadFile.Assign(bookmarkFn);
+		failFile.Assign(bookmarkFn + ".failedload");
+		lastLoaded.Assign(bookmarkFn + ".lastloaded");
+		backupFile.Assign(bookmarkFn + ".backup");	
+	}
+	else {
+		loadFile.Assign(wxGetApp().getConfig()->getConfigDir(), bookmarkFn);
+		failFile.Assign(wxGetApp().getConfig()->getConfigDir(), bookmarkFn + ".failedload");
+		lastLoaded.Assign(wxGetApp().getConfig()->getConfigDir(), bookmarkFn + ".lastloaded");
+		backupFile.Assign(wxGetApp().getConfig()->getConfigDir(), bookmarkFn + ".backup");
+	}
 
     DataTree s;
     bool loadStatusOk = true;
-
-    // Clear any active data
-    bmData.clear();
-    clearRecents();
-    clearRanges();
-    bmDataSorted.clear();
     
     // File exists but is not readable
     if (loadFile.FileExists() && !loadFile.IsFileReadable()) {
@@ -104,6 +137,17 @@ bool BookmarkMgr::loadFromFile(std::string bookmarkFn, bool backup) {
     if (!s.LoadFromFileXML(loadFile.GetFullPath(wxPATH_NATIVE).ToStdString())) {
         return false;
     }
+
+	//Check if it is a bookmark file, read the root node.
+	if (s.rootNode()->getName() != "cubicsdr_bookmarks") {
+		return false;
+	}
+	
+	// Clear any active data
+	bmData.clear();
+	clearRecents();
+	clearRanges();
+	bmDataSorted.clear();
     
     if (s.rootNode()->hasAnother("branches")) {
         DataNode *branches = s.rootNode()->getNext("branches");
@@ -149,7 +193,7 @@ bool BookmarkMgr::loadFromFile(std::string bookmarkFn, bool backup) {
             setExpandState(groupName, (expandState == "true"));
             while (group->hasAnother("modem")) {
                 DataNode *modem = group->getNext("modem");
-                BookmarkEntryPtr be = nodeToBookmark("modem", modem);
+                BookmarkEntryPtr be = nodeToBookmark(modem);
                 if (be) {
                     addBookmark(groupName.c_str(), be);
                 } else {
@@ -165,7 +209,7 @@ bool BookmarkMgr::loadFromFile(std::string bookmarkFn, bool backup) {
         
         while (recent_modems->hasAnother("modem")) {
             DataNode *modem = recent_modems->getNext("modem");
-            BookmarkEntryPtr be = nodeToBookmark("modem", modem);
+            BookmarkEntryPtr be = nodeToBookmark(modem);
             if (be) {
                 addRecent(be);
             } else {
@@ -194,6 +238,17 @@ bool BookmarkMgr::loadFromFile(std::string bookmarkFn, bool backup) {
     return loadStatusOk;
 }
 
+void BookmarkMgr::resetBookmarks() {
+
+	// Clear any active data
+	bmData.clear();
+	clearRecents();
+	clearRanges();
+	bmDataSorted.clear();
+
+	wxGetApp().getAppFrame()->getBookmarkView()->loadDefaultRanges();
+
+}
 
 bool BookmarkMgr::hasLastLoad(std::string bookmarkFn) {
     wxFileName lastLoaded(wxGetApp().getConfig()->getConfigDir(), bookmarkFn + ".lastloaded");
@@ -205,7 +260,7 @@ bool BookmarkMgr::hasBackup(std::string bookmarkFn) {
     return backupFile.FileExists() && backupFile.IsFileReadable();
 }
 
-void BookmarkMgr::addBookmark(std::string group, DemodulatorInstance *demod) {
+void BookmarkMgr::addBookmark(std::string group, DemodulatorInstancePtr demod) {
     std::lock_guard < std::recursive_mutex > lock(busy_lock);
     
 	//Create a BookmarkEntry from demod data, saving its
@@ -360,7 +415,11 @@ bool BookmarkMgr::getExpandState(std::string groupName) {
 
 void BookmarkMgr::updateActiveList() {
 
-	std::lock_guard < std::recursive_mutex > lockData(busy_lock);
+    std::lock_guard < std::recursive_mutex > lockData(busy_lock);
+
+    if (wxGetApp().isShuttingDown()) {
+        return;
+    }
 	
 	BookmarkView *bmv = wxGetApp().getAppFrame()->getBookmarkView();
     
@@ -392,7 +451,7 @@ void BookmarkMgr::updateBookmarks(std::string group) {
 }
 
 
-void BookmarkMgr::addRecent(DemodulatorInstance *demod) {
+void BookmarkMgr::addRecent(DemodulatorInstancePtr demod) {
     std::lock_guard < std::recursive_mutex > lock(busy_lock);
 
     recents.push_back(demodToBookmarkEntry(demod));
@@ -482,7 +541,7 @@ void BookmarkMgr::clearRanges() {
 }
 
 
-BookmarkEntryPtr BookmarkMgr::demodToBookmarkEntry(DemodulatorInstance *demod) {
+BookmarkEntryPtr BookmarkMgr::demodToBookmarkEntry(DemodulatorInstancePtr demod) {
     
     BookmarkEntryPtr be(new BookmarkEntry);
     
@@ -499,7 +558,7 @@ BookmarkEntryPtr BookmarkMgr::demodToBookmarkEntry(DemodulatorInstance *demod) {
     return be;
 }
 
-BookmarkEntryPtr BookmarkMgr::nodeToBookmark(const char *name_in, DataNode *node) {
+BookmarkEntryPtr BookmarkMgr::nodeToBookmark(DataNode *node) {
     if (!node->hasAnother("frequency") || !node->hasAnother("type") || !node->hasAnother("bandwidth")) {
         return nullptr;
     }
@@ -528,26 +587,28 @@ BookmarkEntryPtr BookmarkMgr::nodeToBookmark(const char *name_in, DataNode *node
 std::wstring BookmarkMgr::getBookmarkEntryDisplayName(BookmarkEntryPtr bmEnt) {
     std::wstring dispName = bmEnt->label;
     
-    if (dispName == "") {
+    if (dispName == L"") {
         std::string freqStr = frequencyToStr(bmEnt->frequency) + " " + bmEnt->type;
-        dispName = wstring(freqStr.begin(),freqStr.end());
+
+        dispName = wxString(freqStr).ToStdWstring();
     }
     
     return dispName;
 }
 
-std::wstring BookmarkMgr::getActiveDisplayName(DemodulatorInstance *demod) {
+std::wstring BookmarkMgr::getActiveDisplayName(DemodulatorInstancePtr demod) {
     std::wstring activeName = demod->getDemodulatorUserLabel();
     
-    if (activeName == "") {
+    if (activeName == L"") {
         std::string wstr = frequencyToStr(demod->getFrequency()) + " " + demod->getDemodulatorType();
-        activeName = std::wstring(wstr.begin(),wstr.end());
+
+        activeName = wxString(wstr).ToStdWstring();
     }
     
     return activeName;
 }
 
-void BookmarkMgr::removeActive(DemodulatorInstance *demod) {
+void BookmarkMgr::removeActive(DemodulatorInstancePtr demod) {
 	
 	std::lock_guard < std::recursive_mutex > lock(busy_lock);
 
