@@ -78,7 +78,8 @@ public:
         
         if (name.length() == 0) {
             std::string wstr = frequencyToStr(rangeEnt->startFreq) + " - " + frequencyToStr(rangeEnt->endFreq);
-            name = std::wstring(wstr.begin(),wstr.end());
+
+            name = wxString(wstr).ToStdWstring();
         }
         
         m_questionText->SetLabelText(L"Are you sure you want to remove the range\n '" + name + L"'?");
@@ -103,7 +104,8 @@ public:
         
         if (name.length() == 0) {
             std::string wstr = frequencyToStr(rangeEnt->startFreq) + " - " + frequencyToStr(rangeEnt->endFreq);
-            name = std::wstring(wstr.begin(),wstr.end());
+            
+			name = wxString(wstr).ToStdWstring();
         }
         
         m_questionText->SetLabelText(L"Are you sure you want to update the range\n '" + name + L"' to the active range?");
@@ -311,9 +313,9 @@ wxTreeItemId BookmarkView::refreshBookmarks() {
                 std::wstring fullText = labelVal +
                     L" " + bmEnt->label +
                     L" " + std::to_wstring(bmEnt->frequency) +
-                    L" " + std::wstring(freqStr.begin(),freqStr.end()) +
-                    L" " + std::wstring(bwStr.begin(),bwStr.end()) +
-                    L" " + std::wstring(bmEnt->type.begin(),bmEnt->type.end());
+                    L" " + wxString(freqStr).ToStdWstring() +
+                    L" " + wxString(bwStr).ToStdWstring() +
+                    L" " + wxString(bmEnt->type).ToStdWstring();
                 
                 if (!isKeywordMatch(fullText, searchKeywords)) {
                     continue;
@@ -348,10 +350,9 @@ wxTreeItemId BookmarkView::refreshBookmarks() {
 
 
 void BookmarkView::doUpdateActiveList() {
-    std::vector<DemodulatorInstance *> &demods = wxGetApp().getDemodMgr().getDemodulators();
-    
-//    DemodulatorInstance *activeDemodulator = wxGetApp().getDemodMgr().getActiveDemodulator();
-    DemodulatorInstance *lastActiveDemodulator = wxGetApp().getDemodMgr().getLastActiveDemodulator();
+
+    auto demods = wxGetApp().getDemodMgr().getDemodulators();
+    auto lastActiveDemodulator = wxGetApp().getDemodMgr().getLastActiveDemodulator();
 
     //capture the previously selected item info BY COPY (because the original will be destroyed together with the destroyed tree items) to restore it again after 
     //having rebuilding the whole tree.
@@ -380,9 +381,9 @@ void BookmarkView::doUpdateActiveList() {
             std::wstring fullText = activeLabel.ToStdWstring() +
             L" " + demod_i->getDemodulatorUserLabel() +
             L" " + std::to_wstring(demod_i->getFrequency()) +
-            L" " + std::wstring(freqStr.begin(),freqStr.end()) +
-            L" " + std::wstring(bwStr.begin(),bwStr.end()) +
-            L" " + std::wstring(mtype.begin(),mtype.end());
+            L" " + wxString(freqStr).ToStdWstring() +
+            L" " + wxString(bwStr).ToStdWstring() +
+            L" " + wxString(mtype).ToStdWstring();
             
             if (!isKeywordMatch(fullText, searchKeywords)) {
                 continue;
@@ -419,9 +420,10 @@ void BookmarkView::doUpdateActiveList() {
         
         std::wstring labelVal = re_i->label;
         
-        if (labelVal == "") {
+        if (labelVal == L"") {
             std::string wstr = frequencyToStr(re_i->startFreq) + " - " + frequencyToStr(re_i->endFreq);
-            labelVal = std::wstring(wstr.begin(),wstr.end());
+
+            labelVal = wxString(wstr).ToStdWstring();
         }
         
         wxTreeItemId itm = m_treeView->AppendItem(rangeBranch, labelVal);
@@ -449,9 +451,10 @@ void BookmarkView::doUpdateActiveList() {
         std::wstring labelVal;
         bmr_i->node->child("user_label")->element()->get(labelVal);
 
-        if (labelVal == "") {
-            std::string wstr = frequencyToStr(bmr_i->frequency) + " " + bmr_i->type;
-            labelVal = std::wstring(wstr.begin(),wstr.end());
+        if (labelVal == L"") {
+            std::string str = frequencyToStr(bmr_i->frequency) + " " + bmr_i->type;
+
+            labelVal = wxString(str).ToStdWstring();
         }
         
         if (searchKeywords.size()) {
@@ -461,9 +464,10 @@ void BookmarkView::doUpdateActiveList() {
             
             std::wstring fullText = labelVal +
                 L" " + std::to_wstring(bmr_i->frequency) +
-                L" " + std::wstring(freqStr.begin(),freqStr.end()) +
-                L" " + std::wstring(bwStr.begin(),bwStr.end()) +
-                L" " + std::wstring(bmr_i->type.begin(),tvi->bookmarkEnt->type.end());
+
+                L" " + wxString(freqStr).ToStdWstring() +
+                L" " + wxString(bwStr).ToStdWstring() +
+                L" " + wxString(bmr_i->type).ToStdWstring();
             
             if (!isKeywordMatch(fullText, searchKeywords)) {
                 continue;
@@ -693,7 +697,8 @@ wxButton *BookmarkView::addButton(wxWindow *parent, std::string labelVal, wxObje
 }
 
 
-void BookmarkView::doBookmarkActive(std::string group, DemodulatorInstance *demod) {
+void BookmarkView::doBookmarkActive(std::string group, DemodulatorInstancePtr demod) {
+
     wxGetApp().getBookmarkMgr().addBookmark(group, demod);
     wxGetApp().getBookmarkMgr().updateBookmarks();
 }
@@ -717,7 +722,7 @@ void BookmarkView::doMoveBookmark(BookmarkEntryPtr be, std::string group) {
 }
 
 
-void BookmarkView::doRemoveActive(DemodulatorInstance *demod) {
+void BookmarkView::doRemoveActive(DemodulatorInstancePtr demod) {
 
 	wxGetApp().getBookmarkMgr().removeActive(demod);
 	wxGetApp().getBookmarkMgr().updateActiveList();
@@ -792,7 +797,7 @@ void BookmarkView::onBookmarkChoice( wxCommandEvent & /* event */ ) {
 }
 
 
-void BookmarkView::activeSelection(DemodulatorInstance *dsel) {
+void BookmarkView::activeSelection(DemodulatorInstancePtr dsel) {
     
     m_frequencyVal->SetLabelText(frequencyToStr(dsel->getFrequency()));
     m_bandwidthVal->SetLabelText(frequencyToStr(dsel->getBandwidth()));
@@ -816,8 +821,16 @@ void BookmarkView::activeSelection(DemodulatorInstance *dsel) {
     clearButtons();
 
     addBookmarkChoice(m_buttonPanel);
-    addButton(m_buttonPanel, "Remove Active", wxCommandEventHandler( BookmarkView::onRemoveActive ));
+    
+    if (dsel->isActive() && !(dsel->isRecording())) {
+        addButton(m_buttonPanel, "Start Recording", wxCommandEventHandler( BookmarkView::onStartRecording ));
+    } else {
+        addButton(m_buttonPanel, "Stop Recording", wxCommandEventHandler( BookmarkView::onStopRecording ));
+    }
 
+    addButton(m_buttonPanel, "Remove Active", wxCommandEventHandler( BookmarkView::onRemoveActive ));
+    
+    
     showProps();
     showButtons();
     refreshLayout();
@@ -835,7 +848,7 @@ void BookmarkView::activateBookmark(BookmarkEntryPtr bmEnt) {
 	//the already existing one:
 	// we search among the list of existing demodulators the one matching 
 	//bmEnt and activate it. The search is made backwards, to select the most recently created one.
-	DemodulatorInstance *matchingDemod = wxGetApp().getDemodMgr().getLastDemodulatorWith(
+	DemodulatorInstancePtr matchingDemod = wxGetApp().getDemodMgr().getLastDemodulatorWith(
 																		bmEnt->type,
 																		bmEnt->label, 
 																		bmEnt->frequency, 
@@ -845,7 +858,7 @@ void BookmarkView::activateBookmark(BookmarkEntryPtr bmEnt) {
 
 		matchingDemod = wxGetApp().getDemodMgr().loadInstance(bmEnt->node);
 		matchingDemod->run();
-		wxGetApp().bindDemodulator(matchingDemod);
+		wxGetApp().notifyDemodulatorsChanged();
 	}
 
 	matchingDemod->setActive(true);
@@ -865,7 +878,11 @@ void BookmarkView::activateBookmark(BookmarkEntryPtr bmEnt) {
 
 
 void BookmarkView::activateRange(BookmarkRangeEntryPtr rangeEnt) {
-    wxGetApp().setFrequency(rangeEnt->freq);
+    
+	//the following oly works if rangeEnt->freq is the middle of [rangeEnt->startFreq ; rangeEnt->startFreq]
+	wxGetApp().setFrequency(rangeEnt->freq);
+	
+	// Change View limits to fit the range exactly.
     wxGetApp().getAppFrame()->setViewState(rangeEnt->startFreq + (rangeEnt->endFreq - rangeEnt->startFreq) / 2, rangeEnt->endFreq - rangeEnt->startFreq);
 }
 
@@ -971,7 +988,7 @@ void BookmarkView::rangeSelection(BookmarkRangeEntryPtr re) {
 
     std::string strFreq = frequencyToStr(re->startFreq) + "-" + frequencyToStr(re->endFreq);
     
-    m_frequencyVal->SetLabelText(std::wstring(strFreq.begin(),strFreq.end()));
+    m_frequencyVal->SetLabelText(wxString(strFreq));
     
     showProps();
 
@@ -1149,6 +1166,30 @@ void BookmarkView::onRemoveActive( wxCommandEvent& /* event */ ) {
     }
 }
 
+void BookmarkView::onStartRecording( wxCommandEvent& /* event */ ) {
+    TreeViewItem *curSel = itemToTVI(m_treeView->GetSelection());
+    
+    if (curSel && curSel->type == TreeViewItem::TREEVIEW_ITEM_TYPE_ACTIVE) {
+        if (!curSel->demod->isRecording() && wxGetApp().getConfig()->verifyRecordingPath()) {
+            curSel->demod->setRecording(true);
+            wxGetApp().getBookmarkMgr().updateActiveList();
+        }
+    }
+}
+
+
+void BookmarkView::onStopRecording( wxCommandEvent& /* event */ ) {
+    TreeViewItem *curSel = itemToTVI(m_treeView->GetSelection());
+    
+    if (curSel && curSel->type == TreeViewItem::TREEVIEW_ITEM_TYPE_ACTIVE) {
+        if (curSel->demod->isRecording()) {
+            curSel->demod->setRecording(false);
+            wxGetApp().getBookmarkMgr().updateActiveList();
+        }
+    }
+}
+
+
 
 void BookmarkView::onRemoveBookmark( wxCommandEvent& /* event */ ) {
     if (editingLabel) {
@@ -1411,7 +1452,7 @@ void BookmarkView::onEnterWindow( wxMouseEvent&  event ) {
     }
 #endif
 
-    setStatusText("You can mouse-drag a bookmark entry from one category to the next..etc. TODO: add more Bookmarks descriptions");
+    setStatusText("Drag & Drop to create / move bookmarks, Group and arrange bookmarks, quick Search by keywords.");
 }
 
 
@@ -1473,16 +1514,16 @@ void BookmarkView::onSearchTextFocus( wxMouseEvent&  event ) {
 
 
 void BookmarkView::onSearchText( wxCommandEvent& event ) {
-    wstring searchText = m_searchText->GetValue().Trim().Lower().ToStdWstring();
+    std::wstring searchText = m_searchText->GetValue().Trim().Lower().ToStdWstring();
     
    searchKeywords.clear();
     
    if (searchText.length() != 0) {
         std::wstringstream searchTextLo(searchText);
-        wstring tmp;
+        std::wstring tmp;
         
         while(std::getline(searchTextLo, tmp, L' ')) {
-            if (tmp.length() != 0 && tmp.find(L"search.") == wstring::npos) {
+            if (tmp.length() != 0 && tmp.find(L"search.") == std::wstring::npos) {
                 searchKeywords.push_back(tmp);
 //                std::wcout << L"Keyword: " << tmp << '\n';
             }
@@ -1531,10 +1572,13 @@ void BookmarkView::loadDefaultRanges() {
 
 BookmarkRangeEntryPtr BookmarkView::makeActiveRangeEntry() {
     BookmarkRangeEntryPtr re(new BookmarkRangeEntry);
-    re->freq = wxGetApp().getFrequency();
+   
     re->startFreq = wxGetApp().getAppFrame()->getViewCenterFreq() - (wxGetApp().getAppFrame()->getViewBandwidth()/2);
     re->endFreq = wxGetApp().getAppFrame()->getViewCenterFreq() + (wxGetApp().getAppFrame()->getViewBandwidth()/2);
     
+	//to prevent problems, always make the re->freq the middle of the interval.
+	re->freq = (re->startFreq + re->endFreq) / 2;
+
     return re;
 }
 
